/********************************************************************************
 *   dciBattery -- battery status in PocketPC top bar.                          *
 *                                                                              * 
 *   Copyright (C) 2004-2007 dci (dev@cpp.in)                                   *
 *                                                                              *
 *   This program is free software; you can redistribute it and/or              *
 *   modify it under the terms of the GNU General Public License version 2      *
 *   as published by the Free Software Foundation.                              *
 *                                                                              *
 *   This program is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 *   General Public License version 2 for more details.                         *
 *                                                                              *
 *   You should have received a copy of the GNU General Public License          *
 *   version 2 along with this library; if not, write to the Free Software      *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 ********************************************************************************
 */


#include <windows.h>

#define WCLASS L"dciBatteryCls"
#define BATTERYTIMER 1

HWND hTaskBar = NULL;

int g_iMaxWidth;
int g_iPerc;
int g_iPPh; /* perc per hour */
int g_iSec;


static LRESULT CALLBACK WndProc ( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch ( uMsg )
	{
	case WM_CREATE:
		SetTimer ( hWnd, BATTERYTIMER, 1000, NULL );
		//fall down
	case WM_TIMER:
	{
		int iMyNewWidth;
		RECT rtBar;
		SYSTEM_POWER_STATUS_EX psPower;
		SYSTEMTIME stTime;
		/* load */
		int l_nMaxWidth = g_iMaxWidth;
		int l_nPerc = g_iPerc;
		int l_nSec = g_iSec;
		int l_nPPh = g_iPPh;
		/* width */
		GetClientRect ( GetParent ( hWnd ), &rtBar );
		l_nMaxWidth = rtBar.right - rtBar.bottom;
		/* status */
		GetSystemPowerStatusEx ( &psPower, TRUE );
		l_nPerc = psPower.BatteryLifePercent;
		if ( l_nPerc < 1 ) l_nPerc = 1;
		if ( l_nPerc > 100 ) l_nPerc = 100;
		/* time */
		GetSystemTime ( &stTime );
		l_nSec = ( ( ( ( ( stTime.wDay * 24 ) + stTime.wHour ) * 60 ) + stTime.wMinute ) * 60 ) + stTime.wSecond;
		/* speed */
		if ( g_iPerc > l_nPerc ) /* discharge */
		{
			if ( g_iSec )
			{
				l_nPPh = ( g_iPerc - psPower.BatteryLifePercent ) * ( ( 60 * 60 ) / ( l_nSec - g_iSec ) );
				g_iPPh = l_nPPh;
				g_iSec = 0; /* reset */
			}
		}
		else if ( g_iPerc < l_nPerc ) /* charge */
		{
			l_nPPh = 0;
			g_iPPh = l_nPPh;
			g_iSec = 0; /* reset */
		}
		/* save current clock state, if it had no state */
		if ( 0 == g_iSec ) g_iSec = l_nSec;
		/* calc */
		if ( ( l_nMaxWidth != g_iMaxWidth  ) || ( g_iPerc != l_nPerc ) )
		{
			/* recalc and set width */
			iMyNewWidth = ( l_nPerc * l_nMaxWidth ) / 100;
			SetWindowPos ( hWnd, HWND_TOPMOST, rtBar.bottom, 0, iMyNewWidth, 2, 0 );
			/* redraw */
			InvalidateRect ( hWnd, NULL, TRUE );
			UpdateWindow ( hWnd );
		}
		/* save */
		if ( g_iPerc != l_nPerc )
		{
			g_iPerc = l_nPerc;
			g_iSec = l_nSec;
			g_iPPh = l_nPPh;
		}
		g_iMaxWidth = l_nMaxWidth;
		return 0;
	}
	case WM_PAINT:
	{
		HPEN oPen;
		PAINTSTRUCT ps;
		POINT line[3] = { {1, 0}, {0, 0}, {0, 0} };

		line[2].x = ( ( g_iPerc * g_iMaxWidth ) / 100 ) - 1;
		line[1].x = ( ( g_iPerc - g_iPPh ) * line[2].x ) / g_iPerc; /* overflow may happen! */

		BeginPaint ( hWnd, &ps );
		oPen = ( HPEN ) SelectObject ( ps.hdc, CreatePen ( PS_SOLID, 1, 0x00000000 ) );
		Polyline ( ps.hdc, line, 2 );
		DeleteObject ( SelectObject ( ps.hdc, CreatePen ( PS_SOLID, 1, 0x00C0C0C0 ) ) );
		Polyline ( ps.hdc, line + 1, 2 );
		DeleteObject ( SelectObject ( ps.hdc, oPen ) );
		EndPaint ( hWnd, &ps );
		return 0;
	}
	case WM_DESTROY:
	{
		KillTimer ( hWnd, BATTERYTIMER );
		PostQuitMessage ( 0 );
		return 0;
	}
	default:
		return DefWindowProc ( hWnd, uMsg, wParam, lParam );
	}
}

/* static to save space */
WNDCLASS wc = { CS_HREDRAW | CS_VREDRAW, ( WNDPROC ) WndProc, 0, 0, NULL, NULL, 0, NULL, 0, WCLASS };

/* ep */
int WINAPI WinMain ( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nShowCmd )
{
	if ( hTaskBar = FindWindow ( L"HHTaskBar", NULL ) )
	{
		MSG msg;
		/* running? */
		HWND mWindow = GetWindow ( hTaskBar, GW_CHILD );
		while ( mWindow )
		{
			wchar_t szClassName [ MAX_PATH ];
			GetClassName ( mWindow, szClassName, MAX_PATH );
			if ( 0 == _wcsicmp ( szClassName, WCLASS ) )
			{
				DestroyWindow ( mWindow );
				return 1;
			}
			mWindow = GetWindow ( mWindow, GW_HWNDNEXT );
		}
		/* create class */
		wc.hbrBackground = CreateSolidBrush ( 0x00EEEEEE );
		wc.hInstance = hInstance;
		RegisterClass ( &wc );
		/* create line */
		ShowWindow ( CreateWindow ( WCLASS, NULL, WS_CHILD, 0, 0, 1, 1, hTaskBar, ( HMENU ) NULL, hInstance, NULL ), nShowCmd );
		/* msg loop */
		while ( GetMessage ( &msg, NULL, 0, 0 ) ) DispatchMessage ( &msg );
		/* */
		DeleteObject ( wc.hbrBackground );
		UnregisterClass ( WCLASS, hInstance );
		return msg.wParam;
	}
	else
	{
		MessageBox ( NULL, L"Non PocketPC system?", L"dci", MB_ICONSTOP | MB_TOPMOST );
		return 2;
	}
}
